from __future__ import annotations

import asyncio
import importlib
import os

import discord
from discord.ext import commands

from .bot import StarBot, state_manager
from .db import AbstractDbFactory


async def _main():
    token = os.getenv("DISCORD_TOKEN")
    assert token is not None
    dbfm = os.getenv("DBF_MODULE")
    assert dbfm is not None
    DBF: AbstractDbFactory = importlib.import_module(dbfm).DBF
    async with state_manager(DBF) as state:
        bot = StarBot(state)
        await bot.login(token)
        yield bot
        await bot.load_extension("starbot.stars")
        await bot.connect()
    yield bot


async def aclose(client: discord.Client):
    if not client.is_closed():
        await client.change_presence(status=discord.Status.offline)
        await client.close()


def close(client: discord.Client, loop: asyncio.AbstractEventLoop):
    loop.run_until_complete(aclose(client))


def main():
    loop = asyncio.new_event_loop()
    gen = _main()
    bot: commands.Bot = loop.run_until_complete(anext(gen))
    discord.utils.setup_logging()

    async def complete():
        return await anext(gen)

    task = loop.create_task(complete())
    try:
        loop.run_until_complete(task)
    except (KeyboardInterrupt, InterruptedError, RuntimeError):
        close(bot, loop)
        loop.run_until_complete(task)


if __name__ == "__main__":
    main()
