from __future__ import annotations

from contextlib import AsyncExitStack, asynccontextmanager
from pathlib import Path

import discord
from discord.ext import commands

from .db import AbstractConnection, AbstractDbFactory


class StarState:
    def __init__(self, connection: AbstractConnection) -> None:
        self.connection = connection


@asynccontextmanager
async def state_manager(dbf: AbstractDbFactory):
    async with dbf.from_path(Path("/app/data/stars.db")) as connection:
        yield StarState(connection)


class StarBot(commands.Bot):
    def __init__(self, state: StarState) -> None:
        super().__init__(
            command_prefix="⭐",
            intents=discord.Intents(message_content=True, guild_messages=True, guild_reactions=True, guilds=True),
        )
        self.starstate: StarState = state
